using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Security.Cryptography;
using System.Text;
using System.Windows.Forms;

namespace Soap_Examples
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        /**
         * getToken()  provides a reusable method for creating the token
         * 
         * NOTE:  YOU MUST EDIT THIS METHOD! Set SourceKey and pin to the values
         *  in the source key you created in the merchant console.  If you do not
         *  have a source key,  contact developer support.  YOU WILL NOT BE ABLE
         *  TO RUN THIS CODE WITHOUT A VALID SOURCE KEY.
         **/
        private static usaepay.ueSecurityToken getToken()
        {
            // Instantiate token object
            usaepay.ueSecurityToken token = new usaepay.ueSecurityToken();

            // SourceKey and Pin (created in merchant console)
            token.SourceKey = "P11PON_ENTER_SOURCE_KEY_HERE_KSQr1VT81";
            string pin = "1234";

            // IP address of end user (if applicable)
            token.ClientIP = "11.22.33.44";  

            // Instantiate Hash
            usaepay.ueHash hash = new usaepay.ueHash();
            hash.Type = "md5";  // Type of encryption 
            hash.Seed = Guid.NewGuid().ToString();  // unique encryption seed

            // Assemble string and hash
            string prehashvalue = string.Concat(token.SourceKey, hash.Seed, pin);
            hash.HashValue = GenerateHash(prehashvalue);

            // Add hash to token
            token.PinHash = hash;

            return token;
        }

        /**
         * getClient()  provides a reusable method for creating soap client
         * 
         **/
        private static usaepay.usaepayService getClient()
        {
            // Instantiate soap service client. 'usaepay' is the name you assigned
            //  when adding the web reference
            usaepay.usaepayService client = new usaepay.usaepayService();
            
            // Setup client to use a proxy server (required by some hosting providers)
            //  Uncomment the following lines if you need to use a proxy server
            /*
            System.Net.NetworkCredential proxycreds = new System.Net.NetworkCredential("user", "password");
            System.Net.WebProxy proxy = new System.Net.WebProxy("127.0.0.1", 80); //address of proxy server
            proxy.Credentials = proxycreds;
            client.Proxy = proxy;
            */

            return client;

        }

        private void btnruntransaction_Click(object sender, EventArgs e)
        {
            usaepay.usaepayService client = getClient();
            usaepay.ueSecurityToken token = getToken();

            usaepay.TransactionRequestObject tran = new usaepay.TransactionRequestObject();

            tran.Command = "cc:sale";
            tran.Details =  new usaepay.TransactionDetail();
            tran.Details.Amount = 1.00;
            tran.Details.AmountSpecified = true;
            tran.Details.Invoice = "1234";
            tran.Details.Description = "Example Transaction";

            tran.CreditCardData = new usaepay.CreditCardData();
            tran.CreditCardData.CardNumber = "4444555566667779";
            tran.CreditCardData.CardExpiration = "0909";

            usaepay.TransactionResponse response = new usaepay.TransactionResponse();

            try
            {
                response = client.runTransaction(token, tran);

                if (response.ResultCode == "A")
                {
                    MessageBox.Show(string.Concat("Transaction Approved, RefNum: ",
                            response.RefNum));
                }
                else
                {
                    MessageBox.Show(string.Concat("Transaction Failed: ",
                            response.Error));
                }
            }
            catch (Exception err)
            {
                MessageBox.Show(err.Message);
            }




        }

        private void btnCloseBatch_Click(object sender, EventArgs e)
        {
            usaepay.usaepayService client = getClient();
            usaepay.ueSecurityToken token = getToken();

            Boolean result;

            // close current batch
            try
            {
                result = client.closeBatch(token, "0");

                if (result) MessageBox.Show("Batch closed successfully");
                else MessageBox.Show("Batch failed to close");
            }
            catch (Exception err)
            {
                MessageBox.Show(err.Message);
            }
        }


        private static string GenerateHash(string input)
        {
            // Create a new instance of the MD5CryptoServiceProvider object.
            MD5 md5Hasher = MD5.Create();

            // Convert the input string to a byte array and compute the hash.
            byte[] data = md5Hasher.ComputeHash(Encoding.Default.GetBytes(input));

            // Create a new Stringbuilder to collect the bytes
            // and create a string.
            StringBuilder sBuilder = new StringBuilder();

            // Loop through each byte of the hashed data 
            // and format each one as a hexadecimal string.
            for (int i = 0; i < data.Length; i++)
            {
                sBuilder.Append(data[i].ToString("x2"));
            }

            // Return the hexadecimal string.
            return sBuilder.ToString();
        }




    }
}